from pathlib import Path

import pytest


@pytest.fixture(name="json_stream")
def fixture_json_stream() -> str:
    """Return string with json format"""
    _content = """{"Lorem": {"ipsum": "sit"}}"""
    return _content


@pytest.fixture(name="json_file_path")
def fixture_json_file_path(fixtures_dir: Path) -> Path:
    """Return a file path to example json file"""
    return Path(fixtures_dir, "io", "example_json.json")


@pytest.fixture(name="json_temp_path")
def fixture_json_temp_path(cg_dir: Path) -> Path:
    """Return a temp file path to use when writing json"""
    return Path(cg_dir, "write_json.json")


@pytest.fixture(name="yaml_stream")
def fixture_yaml_stream() -> str:
    """Return string with yaml format"""
    _content = """- Lorem
- ipsum
- sit
- amet
"""
    return _content


@pytest.fixture(name="invalid_yaml_file")
def fixture_invalid_yaml_file(project_dir: Path) -> Path:
    """Return path for invalid yaml file"""
    content = """a: b: c"""
    invalid_yaml: Path = Path(project_dir, "invalid_yaml_file.txt")
    with open(invalid_yaml, "w") as outfile:
        outfile.write(content)
    return invalid_yaml



"""Test ObservationsInputFiles pydantic model behaviour."""

from pathlib import Path

import pytest

from cg.models.observations.input_files import (
    MipDNAObservationsInputFiles,
    validate_observations_file,
)


def test_instantiate_input_files(observations_input_files_raw: dict):
    """Tests input files against a pydantic MipDNAObservationsInputFiles."""

    # GIVEN a dictionary with the basic input files

    # WHEN instantiating an observations input files object
    input_files = MipDNAObservationsInputFiles(**observations_input_files_raw)

    # THEN assert that it was successfully created
    assert isinstance(input_files, MipDNAObservationsInputFiles)


def test_instantiate_input_files_missing_field(
    observations_input_files_raw: dict, file_does_not_exist: Path
):
    """Tests input files against a pydantic MipDNAObservationsInputFiles with not existent field."""

    # GIVEN a dictionary with the basic input files and a file path that does not exist
    observations_input_files_raw["snv_vcf_path"] = file_does_not_exist

    # WHEN checking the observation file

    # THEN the file is not successfully validated and an error is returned
    with pytest.raises(FileNotFoundError):
        # WHEN instantiating a ObservationsInputFiles object
        MipDNAObservationsInputFiles(**observations_input_files_raw)


def test_validate_observations_file(filled_file: Path):
    """Test observations file validation method."""

    # GIVEN a file path

    # WHEN checking the observation file
    observations_file = validate_observations_file(filled_file)

    # THEN the file is successfully validated and returned
    assert observations_file


def test_validate_observations_file_when_no_exists(file_does_not_exist: Path):
    """Test observations file validation method when file does not exist."""

    # GIVEN a file path that does not exist

    # WHEN checking the observation file

    # THEN the file is not successfully validated and an error is returned
    with pytest.raises(FileNotFoundError):
        validate_observations_file(file_does_not_exist)



"""Tests for the formatting module."""

from cg.utils.dict import get_list_from_dictionary


def test_get_list_from_dictionary():
    """Test the formatting of the flattened dictionary."""

    # GIVEN a mock dictionary
    mock_dict = {"a": 1, "b": 2, "c": None, "d": "3"}

    # GIVEN the expected output
    expected_output = ["a", 1, "b", 2, "d", "3"]

    # WHEN calling the function
    flattened_dict = get_list_from_dictionary(mock_dict)

    # THEN assert the output should match the expected list
    assert flattened_dict == expected_output


def test_get_list_from_dictionary_empty_input():
    """Test the formatting of the flattened dictionary."""

    # GIVEN an empty dictionary
    mock_dict = {}

    # WHEN calling the function
    flattened_dict = get_list_from_dictionary(mock_dict)

    # THEN assert the output matches the expected list
    assert not flattened_dict
